@@ -58,8 +58,7 @@ sub list_jobs {
     'select id from minion_jobs
      where (state = $1 or $1 is null) and (task = $2 or $2 is null)
      order by id desc
-     limit $3
-     offset $4', @$options{qw(state task)}, $limit, $offset
+     limit $3 offset $4', @$options{qw(state task)}, $limit, $offset
   )->arrays->map(sub { $self->job_info($_->[0]) })->to_array;
 }
 
@@ -74,8 +73,8 @@ sub list_workers {
 sub new {
   my $self = shift->SUPER::new(pg => Mojo::Pg->new(@_));
 
-  croak 'PostgreSQL 9.4 or later is required'
-    if Mojo::Pg->new(@_)->db->dbh->{pg_server_version} < 90400;
+  croak 'PostgreSQL 9.5 or later is required'
+    if Mojo::Pg->new(@_)->db->dbh->{pg_server_version} < 90500;
   my $pg = $self->pg->auto_migrate(1)->max_connections(1);
   $pg->migrations->name('minion')->from_data;
 
@@ -85,12 +84,12 @@ sub new {
 sub register_worker {
   my ($self, $id) = @_;
 
-  my $sql
-    = 'update minion_workers set notified = now() where id = ? returning 1';
-  return $id if $id && $self->pg->db->query($sql, $id)->rows;
-
-  $sql = 'insert into minion_workers (host, pid) values (?, ?) returning id';
-  return $self->pg->db->query($sql, hostname, $$)->hash->{id};
+  return shift->pg->db->query(
+    "insert into minion_workers (id, host, pid)
+     values (coalesce(?, nextval('minion_workers_id_seq')), ?, ?)
+     on conflict(id) do update set notified = now()
+     returning id", $id, $self->{host} //= hostname, $$
+  )->hash->{id};
 }
 
 sub remove_job {
@@ -148,24 +147,18 @@ sub retry_job {
 sub stats {
   my $self = shift;
 
-  my $db  = $self->pg->db;
-  my $all = $db->query('select count(*) from minion_workers')->array->[0];
-  my $sql
-    = "select count(distinct worker) from minion_jobs where state = 'active'";
-  my $active = $db->query($sql)->array->[0];
+  my $stats = $self->pg->db->query(
+    "select state::text || '_jobs', count(state) from minion_jobs group by state
+     union all
+     select 'inactive_workers', count(*) from minion_workers
+     union all
+     select 'active_workers', count(distinct worker) from minion_jobs
+     where state = 'active'"
+  )->arrays->reduce(sub { $a->{$b->[0]} = $b->[1]; $a }, {});
+  $stats->{inactive_workers} -= $stats->{active_workers};
+  $stats->{"${_}_jobs"} ||= 0 for qw(inactive active failed finished);
 
-  $sql = 'select state, count(state) from minion_jobs group by 1';
-  my $states
-    = $db->query($sql)->arrays->reduce(sub { $a->{$b->[0]} = $b->[1]; $a }, {});
-
-  return {
-    active_jobs => $states->{active} || 0,
-    active_workers   => $active,
-    failed_jobs      => $states->{failed} || 0,
-    finished_jobs    => $states->{finished} || 0,
-    inactive_jobs    => $states->{inactive} || 0,
-    inactive_workers => $all - $active
-  };
+  return $stats;
 }
 
 sub unregister_worker {
@@ -195,7 +188,7 @@ sub _try {
          and task = any (?)
        order by priority desc, created
        limit 1
-       for update
+       for update skip locked
      )
      returning id, args, retries, task", $id,
     $options->{queues} || ['default'], [keys %{$self->minion->tasks}]
@@ -685,7 +678,6 @@ create table if not exists minion_jobs (
   task     text not null,
   worker   bigint
 );
-create index on minion_jobs (priority desc, created);
 create table if not exists minion_workers (
   id      bigserial not null primary key,
   host    text not null,
@@ -735,3 +727,7 @@ alter table minion_jobs alter column result type jsonb using result::jsonb;
 alter table minion_jobs alter column state type text using state;
 alter table minion_jobs alter column state set default 'inactive';
 drop type if exists minion_state;
+
+-- 8 up
+alter table minion_jobs add constraint args check(jsonb_typeof(args) = 'array');
+create index on minion_jobs (state, priority desc, created);
